module Bytemapper
  module NilTimes
    refine NilClass do
      def times
        0
      end
    end
  end

  class Table < Array
    include Flattenable
    include Nameable
    attr_reader :shape, :rows, :bytes
    using NilTimes

    def initialize(shape, rows = nil)
      @shape = Bytemapper.get(shape) || shape
      rows.times { self << @shape  }
    end

    def populate(bytes)
      bytes = bytes.nil? ? '' : bytes
      @bytes = bytes.is_a?(StringIO) ? bytes : StringIO.new(bytes)
      @bytes.string.force_encoding(Encoding::ASCII_8BIT)
      @shape = Bytemapper.get(shape).nil? ? Bytemapper.get2(shape) : Bytemapper.get(shape)

      (bytes.size / shape.size).times { self << @shape } if unbounded?

      table = Table.new(shape) 
      table.clear
      (bytes.size / shape.size).times do
        table << Chunk.new(@bytes.read(shape.size), shape, shape.name) 
      end
      table
    end

    def unbounded?
      empty?
    end

    def size
      empty? ? 0 : map(&:size).reduce(:+)
    end
  end
end
